local _, name_space = ...
local sound_dict = name_space.sound_dict
local player_cooldown = {}
local sound_cooldown = {}
local addon_message_prefix = "SOUNDBOARD"
local current_duration = 0
local current_start = 0

----------------------------UTIL----------------------------

local debug = false
local function dbg_print(msg)
    if(debug) then
        print(msg)
    end
end

function is_retail()
    return floor((floor(select(4, GetBuildInfo()))/10000)) >= 9
end

----------------------------DB----------------------------

local Soundboard = LibStub("AceAddon-3.0"):NewAddon("Soundboard")
local defaults = {
    global = {
        stats = {}
    }
}
function Soundboard:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("SoundboardDB", defaults)
end

local function addStat(sound)
    local old = 0
    if Soundboard.db.global.stats[sound] == nil then
        old = 0
    else
        old = Soundboard.db.global.stats[sound]
    end
    old = old + 1
    Soundboard.db.global.stats[sound] = old
end

local function printStats()
    print("-------SB Stats-------")
    Soundboard.db.global.stats = table.sort(Soundboard.db.global.stats, function (a, b)
        return a < b
    end)

    for key, value in pairs(Soundboard.db.global.stats) do
        print(key, ": ", value)
    end
end

----------------------------PRINT GROUP----------------------------

local function printGroup(group)
    for k,v in pairs(sound_dict) do
        if sound_dict[k].group == group then
            print(k)
        end
    end
end

----------------------------SLASH----------------------------

local groups = {}
for k,v in pairs(sound_dict) do
    groups[sound_dict[k].group] = 1
end
SLASH_SB1 = "/sb"
SlashCmdList["SB"] = function(sound)
    if sound:lower() == "stats" then
        printStats()
        return
    end

    if groups[sound:lower()] == 1 then
        printGroup(sound)
        return
    end

    if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then --Instance Group
        dbg_print("sending " .. sound .. " in INSTANCE_CHAT")
        C_ChatInfo.SendAddonMessage(addon_message_prefix, sound, "INSTANCE_CHAT");
    elseif IsInRaid() and not IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then --Raid Group
        dbg_print("sending " .. sound .. " in RAID")
        C_ChatInfo.SendAddonMessage(addon_message_prefix, sound, "RAID");
    elseif IsInGroup(LE_PARTY_CATEGORY_HOME) then --Party
        dbg_print("sending " .. sound .. " in PARTY")
        C_ChatInfo.SendAddonMessage(addon_message_prefix, sound, "PARTY");
    elseif not is_retail() then
        dbg_print("sending " .. sound .. " in YELL")
        C_ChatInfo.SendAddonMessage(addon_message_prefix, sound, "YELL");
    else
        dbg_print("sending " .. sound .. " in WHISPER")
        name, _ = UnitName("player")
        C_ChatInfo.SendAddonMessage(addon_message_prefix, sound, "WHISPER", name)
    end
end


_G["soundboard_call"] = SlashCmdList["SB"]

----------------------------AUTO-Complete----------------------------

for i=1, NUM_CHAT_WINDOWS do
    local frame = _G["ChatFrame"..i]
    local keyset={}
    local n=0
    
    for k,v in pairs(sound_dict) do
      n=n+1
      keyset[n]="/sb "..k
    end
    
    local editbox = frame.editBox;
    local maxButtonCount = 20;

    local autocompletesettings = {
        perWord = true,
        pattern =  "^/sb .*",
        activationChar = "^/sb .*",
        closingChar = ':',
        minChars = 2,
        fuzzyMatch = true,
        onSuggestionApplied = function(suggestion) return suggestion end,
        renderSuggestionFN = function(text) return text:sub(5, #text) .. " - " .. sound_dict[text:sub(5, #text)].group end,
        suggestionBiasFN = function(suggestion, text) return 0; end,
        interceptOnEnterPressed = true,
        addSpace = true,
        useTabToConfirm = true,
        useArrowButtons = true,
    }
    MySetupAutoComplete(editbox, keyset, maxButtonCount, autocompletesettings);

end
----------------------------QUEUE----------------------------

List = {}
function List.new ()
  return {first = 0, last = -1}
end

function List.pushright (list, value)
    local last = list.last + 1
    list.last = last
    list[last] = value
end

function List.popleft (list)
    local first = list.first
    if first > list.last then error("list is empty") end
    local value = list[first]
    list[first] = nil
    list.first = first + 1
    return value
end

local sound_queue = List:new()

----------------------------EVENTS----------------------------

local function is_on_cooldown(key, list, cooldown)
    if list[key] == nil then
        return false;
    end
    return GetTime() - list[key] < cooldown;
end

local function recvEventHandler(self, event, ...)
    if event ~= "CHAT_MSG_ADDON" then
        return;
    end

    local prefix = select(1, ...);
    local message = select(2, ...);
    local type = select(3, ...);
    local sender = select(4, ...);

    if prefix ~= addon_message_prefix then
        return;
    end

    dbg_print("recvieved " .. message .. " from " .. sender .. " in " .. type .. " chat with " .. prefix .. " prefix")

    --15 sec in group, 60sec in raid
    local cooldown = 15 
    if type == "RAID" then
        cooldown = 60
    elseif debug then
        cooldown = 0
    end

    --if there is no such sound, skip
    local sound_file = sound_dict[message];
    if sound_file == nil then
        return
    end

    --check for cds
    if is_on_cooldown(sender, player_cooldown, cooldown) then
        dbg_print("sender on cooldown")
        return;
    end

    if is_on_cooldown(message, sound_cooldown, cooldown) then
        dbg_print("sound on cooldown")
        return;
    end

    --add cd
    message = message:lower()
    player_cooldown[sender] = GetTime();
    sound_cooldown[message] = GetTime();

    --add sound to q
    dbg_print("adding " .. message .. " to queue")
    addStat(message)
    List.pushright(sound_queue, message);
end

local recvFrame = CreateFrame("Frame");
recvFrame:RegisterEvent("CHAT_MSG_ADDON");
recvFrame:SetScript("OnEvent", recvEventHandler);

local function everyFrameHandler(self, event, ...)
    if sound_queue.first > sound_queue.last then
        return;
    end

    if GetTime() - current_start < current_duration then
        dbg_print("wait until current sound is finished")
        return;
    end

    local command = List.popleft(sound_queue);
    dbg_print("playing " .. command)
    local sound_file = sound_dict[command];
    if sound_file == nil then
        return
    end
    PlaySoundFile(sound_file.path, "Master");
    current_duration = sound_file.duration;
    current_start = GetTime();
end

local everyFrame = CreateFrame("Frame");
everyFrame:SetScript("OnUpdate", everyFrameHandler);
C_ChatInfo.RegisterAddonMessagePrefix(addon_message_prefix);